require(Matrix)

############################
# Lambda.hat and bootstrap #
############################

Lambda.hat<-function(za, zb, dat, mat3=FALSE){

T1<-dat[,1]
D1<-dat[,2]
T2<-dat[,3]
D2<-dat[,4]
S<-dat[,5]
t2<-T2[D2==1]
d1<-D1[D2==1]
s<-S[D2==1]

Lambda.ab<-0; Lambda.ab.t<-NULL; wny.t<-NULL
for (j in order(t2)){
	tj<-t2[j]
	
	y1a<-sum(T2>=tj & T1<=tj & D1==1 & S==za)
	y0a<-sum(T2>=tj & T1>tj & S==za)+sum(T2>=tj & T1<tj & D1==0 & S==za)
	y1b<-sum(T2>=tj & T1<=tj & D1==1 & S==zb)
	y0b<-sum(T2>=tj & T1>tj & S==zb)+sum(T2>=tj & T1<tj & D1==0 & S==zb)
	if (y1a>0){
		dn21a<-d1[j]*(s[j]==za)
		lambda1<-dn21a/y1a
	} else {
		dn21a<-0
		y1a<-0
		lambda1<-0
	}
	if (y0a>0){ 
		dn20a<-(1-d1[j])*(s[j]==za)
		lambda0<-dn20a/y0a
	} else {
		dn20a<-0
		y0a<-0
		lambda0<-0
	}
	if (y1b==0 & y0b==0) {w1<-0.5; w0<-0.5}
	if (y1b==0 & y0b!=0) {w1<-0; w0<-1}
	if (y1b!=0 & y0b==0) {w1<-1; w0<-0}
	if (y1b*y0b!=0) {w1<-y1b/(y1b+y0b); w0<-y0b/(y1b+y0b)}

	lambda.ab<-lambda1*w1+lambda0*w0
	Lambda.ab<-Lambda.ab+lambda.ab*sqrt(y0a*y1a)/(y0a+y1a)
	Lambda.ab.t<-c(Lambda.ab.t, Lambda.ab)
	wny.t<-rbind(wny.t, c(w0, w1, dn20a, dn21a, y0a, y1a, y0b, y1b))
}

mat3.0<-NA
mat3.1<-NA
if (mat3){
	mat3.1<-matrix(0, nc=length(t2), nr=length(t2))
	mat3.0<-matrix(0, nc=length(t2), nr=length(t2))
	for (j in 1:length(t2)){
		tj<-sort(t2)[j]
		for (k in 1:length(t2)){
			tk<-sort(t2)[k]
			mat3.1[j,k]<-sum(T2>=tj & T2>=tk & T1<=tj & T1<=tk & D1==1 & S==zb)
			mat3.0[j,k]<-sum(T2>=tj & T2>=tk & T1>=tj & T1>=tk & S==zb)+0
		}
	}
	mat3.0<-mat3.0*dim(dat)[1]
	mat3.1<-mat3.1*dim(dat)[1]
}

names(Lambda.ab.t)<-sort(t2)
rownames(wny.t)<-sort(t2)
colnames(wny.t)<-c("w0", "w1", "dn20a", "dn21a", "y0a", "y1a", "y0b", "y1b")
out<-list("Lambda.ab.t"=Lambda.ab.t, "wny.t"=wny.t, "m"=dim(dat)[1], "mat3.0"=mat3.0, "mat3.1"=mat3.1)
return(out)
}

Lambda.boot<-function(za, zb, dat, nboot=500){
	Lambda.hat.orig<-Lambda.hat(za, zb, dat)$Lambda.ab.t
	Lambda.hat.boot<-data.frame(matrix(NA, nc=nboot+1, nr=sum(dat[,4]==1)))
	rownames(Lambda.hat.boot)<-names(Lambda.hat.orig)
	Lambda.hat.boot[,1]<-Lambda.hat.orig
	for (b in 1:nboot){
		set.seed(37*b)
		dat.boot<-dat[sample(1:dim(dat)[1], rep=TRUE), ]
		Lambda.hat.b<-Lambda.hat(za, zb, dat.boot)$Lambda.ab.t
		Lambda.hat.boot[names(Lambda.hat.b), b+1]<-Lambda.hat.b
	}
	return(Lambda.hat.boot)
}


mypoly<-function(y1, y2, x, col=rgb(1,0,0,0.5)){
	yy1<-c(rep(y1, each=2)); yy1<-yy1[1:(length(yy1)-1)]
	yy2<-c(rep(y2, each=2)); yy2<-rev(yy2[1:(length(yy2))-1])
	xx<-rep(x, each=2); xx<-xx[2:length(xx)]
	polygon(y=c(yy1, yy2), x=c(xx, rev(xx)), col=col, border=col)
}

################################################################################


###################
# Indirect effect #
###################

IE.hat.var<-function(Lambda.hat.obj0, Lambda.hat.obj1){
	wny0<-as.data.frame(Lambda.hat.obj0$wny.t)
	wny1<-as.data.frame(Lambda.hat.obj1$wny.t)

	v0<-((wny1$w0-wny0$w0)^2)*wny0$dn20a*wny0$y1a/(wny0$y0a*(wny0$y0a+wny0$y1a)^2)
	v0[wny0$y0a^2==0]<-0
	v1<-((wny1$w1-wny0$w1)^2)*wny0$dn21a*wny0$y0a/(wny0$y1a*(wny0$y0a+wny0$y1a)^2)
	v1[wny0$y1a^2==0]<-0
	out<-cumsum(v0+v1)
	names(out)<-rownames(wny0)
	return(out)
}

IE.hat.var2<-function(Lambda.hat.obj){
	wny<-as.data.frame(Lambda.hat.obj$wny.t)
	
	mat3.0<-Lambda.hat.obj$mat3.0
	mat3.1<-Lambda.hat.obj$mat3.1
	m<-Lambda.hat.obj$m
	
	mat1.half<-1/(wny$y0b+wny$y1b)
	mat1.half[is.infinite(mat1.half)]<-0
	mat1<-mat1.half%*%t(mat1.half)
	mat2.0<-wny$y0b%*%t(wny$y0b)
	mat2.1<-wny$y1b%*%t(wny$y1b)
	
	matt0<-(mat3.0-mat2.0)*mat1/m
	matt1<-(mat3.1-mat2.1)*mat1/m
	bandw<-min(20, dim(matt0)[1])
	matt0<-as.matrix(band(matt0, -bandw, bandw))
	matt1<-as.matrix(band(matt1, -bandw, bandw))
	
	vec0<-(wny$dn20a/wny$y0a)*(sqrt(wny$y0a*wny$y1a)/(wny$y0a+wny$y1a))
	vec0[is.nan(vec0)]<-0
	vec1<-(wny$dn21a/wny$y1a)*(sqrt(wny$y0a*wny$y1a)/(wny$y0a+wny$y1a))
	vec1[is.nan(vec1)]<-0
	
	varr<-NULL
	for (j in 1:dim(wny)[1]){
		vec0t<-rep(0, dim(wny)[1])
		vec1t<-rep(0, dim(wny)[1])
		vec0t[1:j]<-vec0[1:j]
		vec1t[1:j]<-vec1[1:j]
		varr.old<-c(varr, t(vec0t)%*%matt0%*%vec0t+t(vec1t)%*%matt1%*%vec1t)
		varr<-varr.old
	}
	return(varr)
}

IE.test<-function(dat){

	temp1<-Lambda.hat(1,1, dat, mat3=TRUE)
	temp0<-Lambda.hat(1,0, dat, mat3=TRUE)
	iehat<-temp1$Lambda.ab.t-temp0$Lambda.ab.t
	
	ie.vartemp<-IE.hat.var(temp0, temp1)
	ie.vartemp2<-IE.hat.var2(temp0)+IE.hat.var2(temp1)
	ie.var<-ie.vartemp+ie.vartemp2
	
	len<-max(which(!is.nan(iehat)))
	
	pval.pt<-pchisq(iehat[len]^2/ie.var[len], df=1, lower.tail=FALSE)
	pval1<-pchisq(iehat[len]^2/ie.vartemp[len], df=1, lower.tail=FALSE)
	pval2<-pchisq(iehat[len]^2/ie.vartemp2[len], df=1, lower.tail=FALSE)
	pval.jt<-max(pval1, pval2)
	return(c(pval.pt, pval.jt, pval1, pval2))
}

